/*
	Author		: Les Farrell (credit me, if you use my code)
	Email  		: lfarrell@btinternet.com
	Compiler 	: LCC-WIN32
	Notes  		: This was my first ever attempt at a windows program
			 	  and as such, you'll see that the code has more than a few rough edges.
			 	  If you find a bit you can use then please do so, all I ask in return
			 	  is a small credit somewhere within you program.

*/

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#include <stdio.h>
#include <math.h>
#include <mmsystem.h>
#include "midi.h"
#include "shellapi.h"
#include "buffer.h"
#include "gamecore.h"
#include "pacman.h"
#include "highscores.h"


// Store some details about when the program was compiled!
const char compile_date[]=__DATE__;
const char compile_time[]=__TIME__;


// WINMAIN
int WINAPI WinMain ( HINSTANCE hInstance, HINSTANCE hprevinstance, LPSTR lpcmdline, int ncmdshow)
{
	WNDCLASS winclass;												// This will hold the class we create
	HWND hwnd;														// Generic window handle
	MSG msg;														// Generic mesage
	HANDLE hAccelTable;


	// Fill in the windows class structure
	if (!hprevinstance)
    {
		winclass.style =CS_BYTEALIGNWINDOW | CS_HREDRAW | CS_VREDRAW;// Set the windows style
		winclass.lpfnWndProc = WindowProc;							// Set the callback routine
		winclass.cbClsExtra = 0;									// Flags
		winclass.cbWndExtra = 0;									// More Flags
		winclass.hInstance = hInstance;								// Current instance
		winclass.hIcon = LoadIcon(hInstance, "MAINICON");			// Windows Icon
		winclass.hCursor = LoadCursor(NULL,IDC_ARROW);				// Default cursor to use
		winclass.hbrBackground = GetStockObject(BLACK_BRUSH);		// Background colour
		winclass.lpszMenuName = "IDR_MENU1";						// Menu
		winclass.lpszClassName = WINDOW_CLASS_NAME;					// Windows Class Name

		if (!RegisterClass(&winclass)) return(0);					// Register our window class
		ThisInstance=hInstance;										// Store the instance value
	}



	// Create the window
	if (!(hwnd = CreateWindow(WINDOW_CLASS_NAME,					// Class
		"A Pacman Clone - By Les Farrell",							// Title
		WS_OVERLAPPEDWINDOW | WS_VISIBLE,							// Flags
		100,50,														// X,Y
		GetSystemMetrics(SM_CXBORDER)*4 + ClientAreaWidth + GetSystemMetrics(SM_CXDLGFRAME)*2,	// Width
		GetSystemMetrics(SM_CYCAPTION) + GetSystemMetrics(SM_CYBORDER)*4 + ClientAreaHeight + GetSystemMetrics(SM_CYMENU) +	GetSystemMetrics(SM_CYDLGFRAME)*2,	// Height
		NULL,														// Handle to parent
		NULL,														// Handle to menu
		hInstance,													// Instance
		NULL))
		)															// Creation parameters
		return(0);

	main_window_handle = hwnd;										// Save the window handle in a global

    SetTimer(hwnd, ID_MYTIMER, 120, NULL);							// Start a timer


	hAccelTable = LoadAccelerators(hInstance,MAKEINTRESOURCE(IDACCEL));

	ShowWindow(hwnd,SW_SHOW);


	while (GetMessage(&msg,NULL,0,0))
	{
		if (!TranslateAccelerator(msg.hwnd,hAccelTable,&msg))
		{
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}
    return msg.wParam;
} // end WinMain



// This is the main message handler of the system
LRESULT CALLBACK WindowProc(HWND hWnd, UINT msg, WPARAM wparam, LPARAM lparam)
{
	PAINTSTRUCT ps;		// Used in WM_PAINT
	HDC hdc;			// handle to client area device context
	char buffer[255];
	char filename[MAX_PATH];
	char *p;
	BOOL i;

	// Find out what the message is
	switch(msg)
	{

		// Process Menu Commands
    	case WM_COMMAND:
        	switch (wparam)
        	{
				// New Game Selected
				case MNUNEWGAME:

					// Stop any tune!
					StopMidi ();

					// Reset the key press value
					VKEY = 0;

					// Find the device context
					hdc = GetDC(hWnd);

					// Initialise the game settings
					InitGame(ThisInstance, hdc,ClientAreaWidth,ClientAreaHeight);

					// Draw everything in the buffer
					DrawLevel(hdc);

					// Stretch the buffer to screen
					StretchBlt(hdc,0,0, ClientAreaWidth, ClientAreaHeight, BUFFER_HDC, 0,0,BUFFER_WIDTH,BUFFER_HEIGHT, SRCCOPY);

					// Release the device context!
					ReleaseDC(hWnd,hdc);

					GAMESTATE=GSGAMEON;
					return(0);

				// Pause Selected!
				case MNUPAUSEGAME:
					switch (GAMESTATE)
					{
						case GSGAMEON:
						    GAMESTATE = GSPAUSED;
							break;
						case GSPAUSED:
							GAMESTATE = GSGAMEON;

							break;
					}
					return(0);


				// Exit Selected!
        		case MNUEXIT:
					// Post a close message to our window
					SendMessage(hWnd, WM_CLOSE, 0, 0L);
        	    	return(0);

				// Help Selected!
				//case MNUHELPCONTENTS:
				//	if (GAMESTATE == GSGAMEON) GAMESTATE = GSPAUSED;
				//	WinHelp(hWnd,"PACMAN.HLP",HELP_CONTENTS,0);
				//	return(0);

				//case MNUVISITWEBSITE:
				//	if (GAMESTATE == GSGAMEON) GAMESTATE = GSPAUSED;
				//	ShellExecute(NULL,"open", "http://www.???", "", "c:\\", SW_SHOWNORMAL);
				//	return(0);

				// Show the version details
				case MNUVERSIONDETAILS:
					strcpy(buffer,"A Pacman Clone for Windows\n");
					strcat(buffer,"Compiled on ");
					strcat(buffer,compile_date);
					strcat(buffer," at ");
					strcat(buffer,compile_time);
					MessageBox(hWnd,buffer,"Version Details",MB_ICONINFORMATION);
					return(0);


				// About Selected!
        		case MNUABOUT:
					// Show the About Dialog on screen!
            		DialogBox( ThisInstance, "IDD_ABOUT", hWnd, (DLGPROC) AboutDlgProc );
      				break;

        	}
        	return(0);


		// Application Active / Deactive
		case WM_ACTIVATEAPP:
		{

			// Application just lost focus so pause the game if needed
			if ((BOOL) wparam == FALSE && GAMESTATE  == GSGAMEON) GAMESTATE = GSPAUSED;

			// Application just got focus so unpause the game.
			// if ((BOOL) wparam == TRUE && GAMESTATE  == 10) GAMESTATE = 0;

			return(0);
		}


		// Menu Activation
		case WM_INITMENU:
		{
			return(0);
		}


		// Window created
		case WM_CREATE:
		{
			// Get the full filename for the application
			GetModuleFileName(ThisInstance,filename,MAX_PATH);

			// Point to the end of the path
			p=filename + strlen(filename);

			// Loop back until we find the first back slash or reach zero
		    while ((DWORD) p > 0 && *p != '\\') p--;

			// Make this the end of the string
        	*++p ='\0';

			// Change to the current Directory!
			SetCurrentDirectory(filename);

			// Do initialization stuff here!
			hdc = GetDC(hWnd);

			InitHighScores();

			// Init the sprite buffer and the double buffer
			InitBuffers(ThisInstance, hdc,ClientAreaWidth,ClientAreaHeight);

			// Init the game settings
			InitGame(ThisInstance, hdc,ClientAreaWidth,ClientAreaHeight);

			StretchBlt(hdc,0,0, ClientAreaWidth, ClientAreaHeight, BUFFER_HDC, 0,0,BUFFER_WIDTH,BUFFER_HEIGHT, BLACKNESS);


			// Release the device context
			ReleaseDC(hWnd,hdc);


			return(0);
		} //break;


		// Process Resize Message, Store the new size of the client area
		case WM_SIZE:
		{
			// Width of new client area
			ClientAreaWidth = LOWORD(lparam);

			// Height of new client area
			ClientAreaHeight = HIWORD(lparam);

			return(0);
		}


		// Window client area needs repainting
		case WM_PAINT:
		{
			// Simply validate the window
			hdc = BeginPaint(hWnd,&ps);

			// Copy the double buffer to the screen
			StretchBlt(hdc,0,0, ClientAreaWidth, ClientAreaHeight, BUFFER_HDC, 0,0,BUFFER_WIDTH,BUFFER_HEIGHT, SRCCOPY);

			// Okay finish painting
			EndPaint(hWnd,&ps);

			return(0);
		}


		// Process Timer Events
		case WM_TIMER:
		{
			// Get the device context
			hdc = GetDC(hWnd);

			// Process the game loop
			GameLoop(hdc,VKEY);

			// Free the device context
			ReleaseDC(hWnd, hdc);

			return(0);
		}


		// About to destroy the window
		case WM_DESTROY:
		{
			// Stop the Timer
        	KillTimer(hWnd, ID_MYTIMER);

			// Kill the application
			PostQuitMessage(0);

			return(0);
		}


		// User Pressed a key store the key value
		case WM_KEYDOWN:
		{
			// Store the last key pressed
			VKEY = (int)wparam;

			return(0);
		}

		case MM_MCINOTIFY:
		{
				//StopMidi ();
				//UnloadMidi ();
				PlayingMIDI = 0;
				break;
		}

		default: break;



	} // End switch

	// Let windows process any messages that we didn't take care of
	return (DefWindowProc(hWnd,msg,wparam,lparam));

} // End WinProc




//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
// Process the 'ABOUT' Dialog
//-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
LRESULT WINAPI AboutDlgProc( HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch( uMsg )
    {

		// Initialise the Dialog box
		case WM_INITDIALOG:
      		return TRUE;

		// Process commands from the dialog
   		case WM_COMMAND:
      		switch( wParam )
		    {
				// User pressed the okay button
         		case IDOK:
					// Terminate the Dialog
            		EndDialog( hwndDlg, TRUE );
            		return TRUE;
      		}
   			break;
	}
	return(FALSE);
}



